## Author:  Xu Zhang
## Date:    September 25th 2017
## Program: This code produces the 2nd tropical PCA of a small randomly 
##          generated dataset with 20 trees and 8 leaves each in the 
##          theoretical algorithm.
## Input:   If you want to apply a different size of dataset, then you'll
##          need to change "DD" and adjust the values of "u" "v" for the
##          upper bounds in the Propostiion 4.2 in our paper; if you want
##          a different order of PC, you need to change "pcs". In this 
##          code, pcs is equal to 3 which is the 2nd PC.
## Output:  "random_sample_20_8.txt" where stores the randomly generated
##          trees; "random_output_20_8.txt".



require(Matrix)
require(ape)
require(cplexAPI)

setwd("~/Dropbox/rudy_xu/sim gene under species")

dist_vector<-function(k,m){
  vecTreesArray<-array(NA,dim = c(k,choose(m,2))) 
  tree_store<-list()
  for(p in 1:k){
    t <- rcoal(m) ### generate an equidistant tree 
    tree_store[[p]]<-t
    t$tip.label<-seq(1,m,by=1)  ###change the labels
    
    D1 <- cophenetic(t) ### Compute the distant matrix
    
    ###vectorize the distance matrix
    for(row.num in 1:(m-1)){
      for(col.num in (row.num+1):m){
        vecTreesArray[p,col.num-row.num+(m-1+(m-1-row.num+2))*(row.num-1)/2]<-D1[row.num,col.num]  
      }
    }   
  }
  return(list(vecTreesArray,tree_store))
}


DD<-dist_vector(20,8)
#par(mfrow=c(2,4))
#plot(DD[[2]][[1]])

class(DD[[2]])<-'multiPhylo'
write.tree(DD[[2]],file='random_sample_20_8.txt')
D<-DD[[1]]
#D<-matrix(c(1.30264066,1.9352151,1.9352151,),nrow=6,byrow = T)
n<-dim(D)[1]
e<-dim(D)[2]
#data_matrix<-cbind(d1,d2,d3) #by column
data_matrix<-t(D)
pcs<-3


#n<-3
#e<-3
#data_matrix<-matrix(c(0,2,5,0,0,0,0,3,1),ncol=n,byrow=F)
#D<-t(data_matrix)
#pcs<-2
###%%%%%%%%%%%%%%%%%

f.obj<-c(rep(1,n),rep(0,n*e+pcs*n+pcs*e+pcs*n*e*2))
f.dir<-c(rep(">=",2*n*e*(e-1)/2),rep("<=",n*pcs*e),rep(">=",n*pcs*e*2),rep("<=",
                                                                           n*pcs*e+n*pcs+n*e),rep("<=",n*e*pcs),rep(">=",n*e*pcs),rep("<=",n*e*pcs),
         rep(">=",n*e*pcs))
diff.dist<-function(d){
  e<-length(d)
  diff.m<-matrix(NA,e,e)
  for(i in 1:(e-1)){
    for(j in (i+1):e){
      diff.m[i,j]<-d[i]-d[j]
    }
  }
  diff.vec<-as.vector(t(diff.m))
  diff.out<-diff.vec[!is.na(diff.vec)]
  return(diff.out)
}

#data.list<-list(D[1,],D[2,],D[3,],D[4,],D[5,],D[6,],
#                D[7,],D[8,]) #data.list<-list(d1,d2,d3)
data.list<-lapply(seq_len(nrow(D)), function(i) D[i,])

#di(k)-di(l),1<=k<l<=e
rhs.1<-as.vector(unlist(lapply(data.list,diff.dist))) 
#-di(k)+di(l),1<=k<l<=e
rhs.2<--rhs.1
rhs.3<-rep(as.vector(data_matrix),pcs) #c(d1,d2,d3,d1,d2,d3)
rhs.4<-rep(0,pcs*n*e)
rhs.5<-rhs.3
f.rhs<-c(rhs.1,rhs.2,rhs.3,rhs.4,rhs.5,rep(0,pcs*n*e),rep(e-1,n*pcs),
         rep(pcs-1,e*n),rep(1,pcs*n*e),rep(0,pcs*n*e),rep(1,pcs*n*e),rep(0,pcs*n*e))

con.m<-replicate(n,rep(1,e*(e-1)/2))
mlist1<-lapply(seq_len(ncol(con.m)), function(i) con.m[,i])
delta_A1<-as.matrix(bdiag(mlist1))
delta_m<-NULL
for(i in 1:(e-1)){
  for(j in (i+1):e){
    pair<-rep(0,e)
    pair[i]<-1
    pair[j]<--1
    delta_m<-rbind(delta_m,pair)
  }
}
delta_A2<-as.matrix(bdiag(replicate(n,list(delta_m))))
delta_A<-cbind(delta_A1,delta_A2,matrix(0,nrow=n*e*(e-1)/2,ncol=pcs*(n+e)+pcs*n*e*2))
delta_AA<-cbind(delta_A1,-delta_A2,matrix(0,nrow=n*e*(e-1)/2,ncol=pcs*(n+e)+pcs*n*e*2))

d_p_A1<-matrix(rep(diag(1,e*n),pcs),ncol=n*e,byrow=TRUE)
con.mm<-replicate(n*pcs,rep(-1,e))
mlist2<-lapply(seq_len(ncol(con.mm)), function(i) con.mm[,i])
d_p_A2<-as.matrix(bdiag(mlist2))
d_p_A3<-as.matrix(bdiag(replicate(pcs,list(matrix(rep(diag(-1,e),n),ncol=e,byrow=TRUE)))))
d_p_A<-cbind(matrix(0,nrow=pcs*e*n,ncol=n),d_p_A1,d_p_A2,d_p_A3,
             matrix(0,nrow=pcs*e*n,ncol=pcs*n*e*2))

d_A<-cbind(matrix(0,nrow=pcs*e*n,ncol=n*(1+e)),-d_p_A2,-d_p_A3,
           matrix(0,nrow=pcs*e*n,ncol=pcs*n*e*2))
v<-200########
d_v<-cbind(matrix(0,nrow=pcs*e*n,ncol=n*(1+e)),-d_p_A2,-d_p_A3,
           v*diag(1,nrow=pcs*n*e,ncol=pcs*n*e),diag(0,nrow=pcs*n*e,ncol=pcs*n*e))
u<-200########
d_p_u<-cbind(matrix(0,nrow=pcs*e*n,ncol=n),d_p_A1,d_p_A2,d_p_A3,
             diag(0,nrow=pcs*n*e,ncol=pcs*n*e),-u*diag(1,nrow=pcs*n*e,ncol=pcs*n*e))

con.mz<-matrix(1,nrow=n*pcs,ncol=e)
mlist3<-lapply(seq_len(nrow(con.mz)), function(i) con.mz[i,])
m_z1<-as.matrix(t(bdiag(mlist3)))
m_z<-cbind(matrix(0,nrow=pcs*n,ncol=n+n*e+pcs*n+pcs*e),
           m_z1,diag(0,nrow=pcs*n,ncol=pcs*n*e))

m_y1<-do.call("cbind", rep(list(diag(1,n*e)), pcs))
m_y<-cbind(matrix(0,nrow=e*n,ncol=n+n*e+pcs*n+pcs*e),
           diag(0,nrow=e*n,ncol=pcs*n*e),m_y1)
bound_z<-cbind(matrix(0,nrow=pcs*n*e,ncol=n+n*e+pcs*n+pcs*e),
               diag(1,pcs*n*e),diag(0,nrow=pcs*n*e,ncol=pcs*n*e))
bound_y<-cbind(matrix(0,nrow=pcs*n*e,ncol=n+n*e+pcs*n+pcs*e),
               diag(0,nrow=pcs*n*e,ncol=pcs*n*e),diag(1,pcs*n*e))
f.con<-rbind(delta_A,delta_AA,d_A,d_p_A,d_v,d_p_u,m_z,m_y,
             bound_z,bound_z,bound_y,bound_y)

f.int<-seq(n+n*e+pcs*n+pcs*e+1,n+n*e+pcs*n+pcs*e+pcs*n*e*2,by=1)

#library(glpkAPI)
#lp <- initProbGLPK()
nrows  <- dim(f.con)[1]-pcs*n*e*4
ncols  <- dim(f.con)[2]

# direction of optimization
#setObjDirGLPK(lp, GLP_MIN)

# add rows and columns
#addRowsGLPK(lp, nrows)
#addColsGLPK(lp, ncols)

#Set the type and bounds of rows
lb<-c(rhs.1,rhs.2,rep(0,length(rhs.3)),rhs.4,rhs.5,rep(0,pcs*n*e),rep(0,n*pcs),
      rep(0,e*n))
ub<-c(rep(0,length(rhs.1)*2),rhs.3,rep(0,length(rhs.4)),rep(0,length(rhs.5)),
      rep(0,pcs*n*e),rep(e-1,n*pcs),rep(pcs-1,e*n))
#type<-c(rep(GLP_LO,length(rhs.1)*2),rep(GLP_UP,length(rhs.3)),rep(GLP_LO,length(rhs.4)),
#        rep(GLP_LO,length(rhs.5)),rep(GLP_UP,pcs*n*e),rep(GLP_UP,n*pcs),rep(GLP_UP,e*n))
#setRowsBndsGLPK(lp,1:nrows,lb,ub,type)

#Set the type and bounds of columns and the objective function
lb_c<-c(rep(0,n+n*e+pcs*n+pcs*e),rep(0,pcs*n*e*2))
ub_c<-c(rep(0,n+n*e+pcs*n+pcs*e),rep(1,pcs*n*e*2))
#type_c<-c(rep(GLP_FR,n+n*e+pcs*n+pcs*e),rep(GLP_DB,pcs*n*e*2))
obj<-c(rep(1,n),rep(0,n*e+pcs*n+pcs*e+pcs*n*e*2))
#setColsBndsObjCoefsGLPK(lp, 1:ncols, lb_c, ub_c, obj, type_c)
#setColsKindGLPK(lp,(n+n*e+pcs*n+pcs*e+1):ncols,rep(GLP_IV,pcs*e*n*2))

#Load the constraint matrix
ia<-rep(seq(1,nrows,1),ncols)
ja<-as.vector(t(replicate(nrows,seq(1,ncols,1))))
ar<-as.vector(rbind(delta_A,delta_AA,d_A,d_p_A,d_v,d_p_u,m_z,m_y))
#loadMatrixGLPK(lp, nrows*ncols, ia, ja, ar)

#setDefaultMIPParmGLPK()
#setMIPParmGLPK(PRESOLVE, GLP_ON)
#solveMIPGLPK(lp)
#mipObjValGLPK(lp)
#out_put<-mipColsValGLPK(lp)
#out_D_1<-out_put[(n+n*e+n*pcs+1):(n+n*e+n*pcs+e)]
#out_D_2<-out_put[(n+n*e+n*pcs++e+1):(n+n*e+n*pcs+2*e)]
#out_D_3<-out_put[(n+n*e+n*pcs+2*e+1):(n+n*e+n*pcs+3*e)]

#library(cplexAPI)

#update the variables and constraints
obj_base<-c(rep(1,n),rep(0,n*e+pcs*n+pcs*e+pcs*n*e*2))


con_final<-rbind(delta_A,delta_AA,d_A,d_p_A,d_v,d_p_u,m_z,m_y)
# Open a CPLEX environment
env <- openEnvCPLEX()
# Create a problem object
prob <- initProbCPLEX(env)
# Assign a name to the problem object
# Prepare data structures for the problem object
# Number of columns and rows
nr  <- dim(f.con)[1]-pcs*n*e*4
nc  <- dim(f.con)[2]
nz <-nr*nc

# Objective function
obj<-obj_base
# Right hand side
rhs <- c(rhs.1,rhs.2,rhs.3,rhs.4,rhs.5,rep(0,pcs*n*e),rep(e-1,n*pcs),rep(pcs-1,e*n))
# Sense of the right hand side
sense <- c(rep("G",length(rhs.1)*2),rep("L",length(rhs.3)),rep("G",length(rhs.4)),
           rep("G",length(rhs.5)),rep("L",pcs*n*e),rep("L",n*pcs),rep("L",e*n))
ctype<-c(rep("C", n),rep("C",n*e+pcs*n+pcs*e),rep(CPX_BINARY,n*e*pcs*2))
# Variable lower bounds
lb <- c(rep(0, n),rep(-10000000,n*e+pcs*n+pcs*e),rep(0,n*e*pcs*2))
# Variable upper bounds
ub <-c(rep(CPX_INFBOUND, n),rep(CPX_INFBOUND,n*e+pcs*n+pcs*e),rep(1,n*e*pcs*2)) 
#column and row names
cn<-seq(1,nc,by=1)
rn<-seq(1,nr,by=1)

beg<-seq(0,nz-nc,by=nr)
cnt<-rep(nr,nc)
ind<-rep(seq(0,nr-1,by=1),nc)
val<-as.vector(con_final)
copyLpCPLEX(env, prob, nc, nr, CPX_MIN, obj, rhs, sense, beg, cnt, ind, val, lb, ub)
copyColTypeCPLEX(env, prob, ctype)
# Solve the MIP problem
setIntParmCPLEX(env, CPX_PARAM_SCRIND, CPX_ON)
setDblParmCPLEX(env, CPX_PARAM_EPRHS, 1E-9)

#setIntParmCPLEX(env, CPXPARAM_Advance, 1)
#untit5<-cplex_initial
#addMIPstartsCPLEX(env,prob,1,nc,beg=0,varindices = seq(0,nc-1,by=1),
#                  values = untit5,effortlevel = 1)

mipoptCPLEX(env, prob)
# Retrieve solution after optimization
solutionCPLEX(env, prob)

out_put<-solutionCPLEX(env, prob)
out_D_1<-out_put$x[(n+n*e+n*pcs+1):(n+n*e+n*pcs+e)]
out_D_2<-out_put$x[(n+n*e+n*pcs+e+1):(n+n*e+n*pcs+2*e)]
out_D_3<-out_put$x[(n+n*e+n*pcs+2*e+1):(n+n*e+n*pcs+3*e)]

proj_matrix<-matrix(out_put$x[(n+1):(n+n*e)],nrow=n,ncol=e,byrow=T)

print.list<-list(data_matrix,out_put$objval,cbind(out_D_1-min(out_D_1),out_D_2-min(out_D_2),out_D_3-min(out_D_3)),
                 t(proj_matrix))
names(print.list)<-c("Random trees","Optimized objective value",
                     "PC's","Projected trees")
sink("random_output_20_8.txt")
print(print.list)
#print(list(data_matrix,out_put$objval,cbind(out_D_1-min(out_D_1),out_D_2-min(out_D_2),out_D_3-min(out_D_3)),
#           t(proj_matrix)))
sink()
#getStatCPLEX()
#getObjValCPLEX(env,prob)
#getProbVarCPLEX
delProbCPLEX(env, prob)
closeEnvCPLEX(env)






# # Open a CPLEX environment
# env2 <- openEnvCPLEX()
# # Create a problem object
# prob2 <- initProbCPLEX(env2)
# # Assign a name to the problem object
# # Prepare data structures for the problem object
# # Number of columns and rows
# nr2  <- dim(f.con)[1]-pcs*n*e*4
# nc2 <- dim(f.con)[2]
# nz2 <-nr*nc
# 
# # Objective function
# obj2<-c(rep(1,n),rep(0,n*e+pcs*n+pcs*e+pcs*n*e*2))
# # Right hand side
# rhs2 <- c(rhs.1,rhs.2,rhs.3,rhs.4,rhs.5,rep(0,pcs*n*e),rep(e-1,n*pcs),rep(1,e*n))
# # Sense of the right hand side
# sense2 <- c(rep("G",length(rhs.1)*2),rep("L",length(rhs.3)),rep("G",length(rhs.4)),
#            rep("G",length(rhs.5)),rep("L",pcs*n*e),rep("L",n*pcs),rep("L",e*n))
# ctype2<-c(rep("C", n),rep("C",n*e+pcs*n+pcs*e),rep("C",n*e*pcs*2))
# # Variable lower bounds
# lb2 <- c(rep(0, n),rep(-10000000,n*e+pcs*n+pcs*e),rep(0,n*e*pcs*2))
# # Variable upper bounds
# ub2 <-c(rep(CPX_INFBOUND, n),rep(CPX_INFBOUND,n*e+pcs*n+pcs*e),rep(1,n*e*pcs*2)) 
# #column and row names
# cn2<-seq(1,nc,by=1)
# rn2<-seq(1,nr,by=1)
# 
# beg2<-seq(0,nz-nc,by=nr)
# cnt2<-rep(nr,nc)
# ind2<-rep(seq(0,nr-1,by=1),nc)
# val2<-ar
# copyLpwNamesCPLEX(env2, prob2, nc2, nr2, CPX_MIN, obj2, rhs2, sense2, beg2, cnt2, ind2, val2, lb2, ub2)
# #copyColTypeCPLEX(env2, prob2, ctype2)
# # Solve the LP problem
# setIntParmCPLEX(env2, CPX_PARAM_SCRIND, CPX_ON)
# #setDblParmCPLEX(env2, CPX_PARAM_EPRHS, 1E-9)
# lpoptCPLEX(env2, prob2)
# # Retrieve solution after optimization
# solutionCPLEX(env2, prob2)
# 
# out_putLp<-solutionCPLEX(env2, prob2)
# out_D_1Lp<-out_put[(n+n*e+n*pcs+1):(n+n*e+n*pcs+e)]
# out_D_2Lp<-out_put[(n+n*e+n*pcs++e+1):(n+n*e+n*pcs+2*e)]
# out_D_3Lp<-out_put[(n+n*e+n*pcs+2*e+1):(n+n*e+n*pcs+3*e)]
# #getStatCPLEX()
# #getObjValCPLEX(env,prob)
# #getProbVarCPLEX
# delProbCPLEX(env2, prob2)
# closeEnvCPLEX(env2)
# 
# 
# 

# 
# obj_base<-c(rep(1,n),rep(0,n*e+pcs*n+pcs*e+pcs*n*e*2),rep(1,3))#add s12,s13,s23
# 
# #delta_A,delta_AA,d_A,d_p_A,d_v,d_p_u,m_z,m_y
# con_m_update<-cbind(rbind(delta_A,delta_AA,d_A,d_p_A,d_v,d_p_u,m_z,m_y),
#                     matrix(0,nrow=dim(f.con)[1]-pcs*n*e*4,ncol=3))
# 
# unequal_base<-delta_A[1:(e*(e-1)/2),(n+1):(n+e)]
# unequal_m12<-cbind(matrix(0,nrow =e*(e-1)/2,ncol = n+n*e+pcs*n ),-unequal_base,unequal_base,
#                    matrix(0,nrow =e*(e-1)/2,ncol = e+n*e*pcs*2),rep(1,e*(e-1)/2),rep(0,e*(e-1)/2),rep(0,e*(e-1)/2))
# unequal_m12_2<-cbind(matrix(0,nrow =e*(e-1)/2,ncol = n+n*e+pcs*n ),unequal_base,-unequal_base,
#                      matrix(0,nrow =e*(e-1)/2,ncol = e+n*e*pcs*2),rep(1,e*(e-1)/2),rep(0,e*(e-1)/2),rep(0,e*(e-1)/2))
# unequal_m13<-cbind(matrix(0,nrow =e*(e-1)/2,ncol = n+n*e+pcs*n ),-unequal_base,
#                    matrix(0,nrow =e*(e-1)/2,ncol = e),unequal_base,
#                    matrix(0,nrow =e*(e-1)/2,ncol = n*e*pcs*2),rep(0,e*(e-1)/2),rep(1,e*(e-1)/2),rep(0,e*(e-1)/2))
# unequal_m13_2<-cbind(matrix(0,nrow =e*(e-1)/2,ncol = n+n*e+pcs*n ),unequal_base,
#                      matrix(0,nrow =e*(e-1)/2,ncol = e),-unequal_base,
#                      matrix(0,nrow =e*(e-1)/2,ncol = n*e*pcs*2),rep(0,e*(e-1)/2),rep(1,e*(e-1)/2),rep(0,e*(e-1)/2))
# unequal_m23<-cbind(matrix(0,nrow =e*(e-1)/2,ncol = n+n*e+pcs*n ),
#                    matrix(0,nrow =e*(e-1)/2,ncol = e),-unequal_base,unequal_base,
#                    matrix(0,nrow =e*(e-1)/2,ncol = n*e*pcs*2),rep(0,e*(e-1)/2),rep(0,e*(e-1)/2),rep(1,e*(e-1)/2))
# unequal_m23_2<-cbind(matrix(0,nrow =e*(e-1)/2,ncol = n+n*e+pcs*n ),
#                      matrix(0,nrow =e*(e-1)/2,ncol = e),unequal_base,-unequal_base,
#                      matrix(0,nrow =e*(e-1)/2,ncol = n*e*pcs*2),rep(0,e*(e-1)/2),rep(0,e*(e-1)/2),rep(1,e*(e-1)/2))
# 
# con_final<-rbind(con_m_update,unequal_m12,unequal_m12_2,unequal_m13,unequal_m13_2,unequal_m23,unequal_m23_2)
# 
# env <- openEnvCPLEX()
# # Create a problem object
# prob <- initProbCPLEX(env)
# # Assign a name to the problem object
# # Prepare data structures for the problem object
# # Number of columns and rows
# nr  <- dim(f.con)[1]-pcs*n*e*4
# nc  <- dim(f.con)[2]
# nz <-nr*nc
# 
# # Objective function
# obj<-c(rep(1,n),rep(0,n*e+pcs*n+pcs*e+pcs*n*e*2))
# # Right hand side
# rhs <- c(rhs.1,rhs.2,rhs.3,rhs.4,rhs.5,rep(0,pcs*n*e),rep(e-1,n*pcs),rep(1,e*n)##starting the unequal constraints
#          )
# # Sense of the right hand side
# sense <- c(rep("G",length(rhs.1)*2),rep("L",length(rhs.3)),rep("G",length(rhs.4)),
#            rep("G",length(rhs.5)),rep("L",pcs*n*e),rep("L",n*pcs),rep("L",e*n))
# ctype<-c(rep("C", n),rep("C",n*e+pcs*n+pcs*e),rep(CPX_BINARY,n*e*pcs*2))
# # Variable lower bounds
# lb <- c(rep(0, n),rep(-10000000,n*e+pcs*n+pcs*e),rep(0,n*e*pcs*2))
# # Variable upper bounds
# ub <-c(rep(CPX_INFBOUND, n),rep(CPX_INFBOUND,n*e+pcs*n+pcs*e),rep(1,n*e*pcs*2)) 
# #column and row names
# cn<-seq(1,nc,by=1)
# rn<-seq(1,nr,by=1)
# 
# beg<-seq(0,nz-nc,by=nr)
# cnt<-rep(nr,nc)
# ind<-rep(seq(0,nr-1,by=1),nc)
# 
# val<-as.vector(rbind(delta_A,delta_AA,d_A,d_p_A,d_v,d_p_u,m_z,m_y
#                      #t(c(rep(0,n+n*e+pcs*n),1,-1,rep(0,e-2),-1,1,rep(0,e-2),rep(0,pcs*n*e*2))),
#                      ))
# copyLpCPLEX(env, prob, nc, nr, CPX_MIN, obj, rhs, sense, beg, cnt, ind, val, lb, ub)
# copyColTypeCPLEX(env, prob, ctype)
# # Solve the MIP problem
# setIntParmCPLEX(env, CPX_PARAM_SCRIND, CPX_ON)
# setDblParmCPLEX(env, CPX_PARAM_EPRHS, 1E-9)
# mipoptCPLEX(env, prob)
# # Retrieve solution after optimization
# solutionCPLEX(env, prob)
# 
# out_put<-solutionCPLEX(env, prob)
# out_D_1<-out_put$x[(n+n*e+n*pcs+1):(n+n*e+n*pcs+e)]
# out_D_2<-out_put$x[(n+n*e+n*pcs++e+1):(n+n*e+n*pcs+2*e)]
# out_D_3<-out_put$x[(n+n*e+n*pcs+2*e+1):(n+n*e+n*pcs+3*e)]
# #getStatCPLEX()
# #getObjValCPLEX(env,prob)
# #getProbVarCPLEX
# delProbCPLEX(env, prob)
# closeEnvCPLEX(env)
# 
# 
# env2 <- openEnvCPLEX()
# # Create a problem object
# prob2 <- initProbCPLEX(env2)
# # Assign a name to the problem object
# # Prepare data structures for the problem object
# # Number of columns and rows
# nr  <- dim(f.con)[1]-pcs*n*e*4+3
# nc  <- dim(f.con)[2]
# nz <-nr*nc
# 
# # Objective function
# obj<-c(rep(1,n),rep(0,n*e+pcs*n+pcs*e+pcs*n*e*2))
# # Right hand side
# rhs <- c(rhs.1,rhs.2,rhs.3,rhs.4,rhs.5,rep(0,pcs*n*e),rep(e-1,n*pcs),rep(1,e*n),##starting the unequal constraints
#          rep(0.000001,3))
# # Sense of the right hand side
# sense <- c(rep("G",length(rhs.1)*2),rep("L",length(rhs.3)),rep("G",length(rhs.4)),
#            rep("G",length(rhs.5)),rep("L",pcs*n*e),rep("L",n*pcs),rep("L",e*n),##unequal
#            rep("G",3))
# ctype<-c(rep("C", n),rep("C",n*e+pcs*n+pcs*e),rep("C",n*e*pcs*2))
# # Variable lower bounds
# lb <- c(rep(0, n),rep(-10000000,n*e+pcs*n+pcs*e),rep(0,n*e*pcs*2))
# # Variable upper bounds
# ub <-c(rep(CPX_INFBOUND, n),rep(CPX_INFBOUND,n*e+pcs*n+pcs*e),rep(1,n*e*pcs*2)) 
# #column and row names
# cn<-seq(1,nc,by=1)
# rn<-seq(1,nr,by=1)
# 
# beg<-seq(0,nz-nc,by=nr)
# cnt<-rep(nr,nc)
# ind<-rep(seq(0,nr-1,by=1),nc)
# 
# val<-as.vector(rbind(delta_A,delta_AA,d_A,d_p_A,d_v,d_p_u,m_z,m_y,
#                      t(c(rep(0,n+n*e+pcs*n),1,-1,rep(0,e-2),-1,1,rep(0,e-2),rep(0,e),rep(0,pcs*n*e*2))),
#                      t(c(rep(0,n+n*e+pcs*n),1,-1,rep(0,e-2),0,rep(0,e-1),-1,1,rep(0,e-2),rep(0,pcs*n*e*2))),
#                      t(c(rep(0,n+n*e+pcs*n),0,rep(0,e-1),1,-1,rep(0,e-2),-1,1,rep(0,e-2),rep(0,pcs*n*e*2)))))
# copyLpwNamesCPLEX(env2, prob2, nc, nr, CPX_MIN, obj, rhs, sense, beg, cnt, ind, val, lb, ub)
# 
# # Solve the MIP problem
# setIntParmCPLEX(env2, CPX_PARAM_SCRIND, CPX_ON)
# #setDblParmCPLEX(env2, CPX_PARAM_EPRHS, 1E-9)
# lpoptCPLEX(env2, prob2)
# # Retrieve solution after optimization
# solutionCPLEX(env2, prob2)
# #getStatCPLEX()
# #getObjValCPLEX(env,prob)
# #getProbVarCPLEX
# delProbCPLEX(env2, prob2)
# closeEnvCPLEX(env2)

