## Author:  Xu Zhang
## Date:    September 25th 2017
## Program: This code produces the 2nd tropical PCA of simulation dataset in 
##          the heuristic algorithm.
## Input:   Simulation dataset (If you want to apply a different dataset, then
##          you'll need to change the path to read the data and adjust 
##          "sum_orig"; if you want a different order of PC, you need to change
##          "pcs". In this code, pcs is equal to 3 which is the 2nd PC.)
## Output:  "sim_output_approx_250_8.txt", "proj_sim_250_8_topology.pdf" and 
##          the summary r statistic.


require(Matrix)
require(ape)

setwd("~/Dropbox/rudy_xu/sim gene under species")

species<-read.nexus("species_250_8")
species_chrono<-chronos(species)
D_spe<-cophenetic(species_chrono)

####check for the umtrametrics
#for(i in 1:8){
 # for(j in 1:8){
#    for(k in 1:8){
#      if((D_spe[i,j]<=max(D_spe[i,k],D_spe[k,j])&&
#       D_spe[i,k]<=max(D_spe[i,j],D_spe[k,j])&&
#       D_spe[k,j]<=max(D_spe[i,k],D_spe[i,j]))){print("")}
#      else{print(k)}
#    }
#  }
#}
#####
genes<-read.nexus("genes_250_8")

#chronotrees<-list()
genes_chrono<-lapply(genes, chronos)

D_list_gen<-lapply(genes_chrono,cophenetic)


vec_fun<-function(x){
    m<-dim(x)[1]
    vecTreesVec<-rep(NA,choose(m,2)) 
    for(row.num in 1:(m-1)){
        for(col.num in (row.num+1):m){
            vecTreesVec[col.num-row.num+(m-1+(m-1-row.num+2))*(row.num-1)/2]<-x[row.num,col.num]  
        }
    }
    vecTreesVec
}

D_gen<-lapply(D_list_gen,vec_fun)

n<-250
e<-length(D_gen[[1]])
data_matrix<-matrix(unlist(D_gen),ncol=n,byrow=F)
D<-t(data_matrix)
pcs<-3
m<-8

#function to get tropical distance of two points
tropical_dist<-function(D_1,D_2){
  e<-length(D_1)
  t_dist<-0
  for(i in 1:(e-1)){
    for(j in (i+1):e){
      if(abs(D_1[i]-D_2[i]-D_1[j]+D_2[j])>t_dist){
        t_dist<-abs(D_1[i]-D_2[i]-D_1[j]+D_2[j])
      }
    }
  }
  t_dist
}   

#function to project tree D onto tconv(D_s) with s PC's and m leaves
project_pi<-function(s,m,D_s,D){
  lambda<-rep(NA,s)
  for(i in 1:s){
    lambda[i]<-min(D-D_s[i,])#D_s by row
  }
  
  pi_D<-rep(NA,choose(m,2))
  for(i in 1:choose(m,2)){
    pi_D[i]<-max(lambda+D_s[,i])
  }
  return(pi_D)
}

#DD<-dist_vector(20,8)
#par(mfrow=c(2,4))
#plot(DD[[2]][[1]])

#class(DD[[2]])<-'multiPhylo'
#write.tree(DD[[2]],file='random_sample_20_8.txt')
#D<-DD[[1]]
#n<-dim(D)[1]
#e<-dim(D)[2]
#data_matrix<-cbind(d1,d2,d3) #by column
#data_matrix<-t(D)
#pcs<-3
#m<-8 #num of leaves

#n<-5
#e<-3
#data_matrix<-matrix(c(1,3,3,0,1,1,0,3,3,3.5,4,4,2,7,7),ncol=n,byrow=F)
#D<-t(data_matrix)
#pcs<-3
###%%%%%%%%%%%%%%%%%

#format(round(D[1,1]-D[1,2],10),nsmall=10)

#D<-matrix(c(
#0.79079649, 2.5696926, 2.5696926
#,0.39710295, 0.6266248, 0.6266248
#,0.26087805, 2.7392306 ,2.7392306
#,3.10106924, 4.0755676 ,4.0755676
#,2.01038059 ,7.4739442, 7.4739442
#,0.02879996, 3.7041242, 3.7041242),nrow=6,byrow=T)

#all possible combinations
sample_set_ind<-combn(n,pcs,simplify = F)
sum_orig<-10000000
data_new<-D #not exclude the PC base
data.new.list<-lapply(seq_len(nrow(data_new)), function(j) data_new[j,])
for(i in 1:choose(n,pcs)){
  pc_base<-D[sample_set_ind[[i]],]#by row
  proj_points<-lapply(data.new.list,project_pi,s=pcs,m=m,D_s=pc_base)
  tropical.dist.vec<-mapply(tropical_dist,data.new.list,proj_points)
  sum_dist<-sum(tropical.dist.vec)
  if(sum_dist < sum_orig){
    label_output<-list(pc_base,proj_points,tropical.dist.vec,sum_dist,i)
    sum_orig<-sum_dist
  }
  print(i)
}
names(label_output)<-c("PC's","Projected points","Dist","Sum of dist","i")

sink("sim_output_approx_250_8.txt")
print(label_output)
sink()



###########################
library(phangorn)

make.matrix <- function(D, n, tips){
  dd <- matrix(rep(0, n*n), nrow=n, ncol=n, byrow=TRUE)
  count <- 1
  for(i in 1:(n-1))
    for(j in (i + 1):n){
      dd[i, j] <- D[count]
      dd[j, i] <- dd[i, j]
      count <- count+1
    }
  mymatrix <- matrix(dd, nrow=n, ncol=n, byrow=TRUE, dimnames=list(tips,tips))
  return(mymatrix)
}

numvectors <- dim(data_matrix)[2]

adj_proj_points<-label_output$`Projected points`
adj_proj_points[[4]]<-adj_proj_points[[4]]+2
adj_proj_points[[11]]<-adj_proj_points[[11]]+2
adj_proj_points[[13]]<-adj_proj_points[[13]]+2

#proj_trees_matrix<-lapply(label_output$`Projected points`,make.matrix,
#                  n=m,tips=chronotrees[[1]]$tip.label)
proj_trees_matrix<-lapply(adj_proj_points,make.matrix,
                          n=m,tips=genes[[1]]$tip.label)
proj_trees<-lapply(proj_trees_matrix,upgma)
type <- seq(numvectors)
for(i in seq(numvectors)){
  if(i==type[i]) {
    for(j in seq(i, numvectors)) {
      if(RF.dist(proj_trees[[i]],proj_trees[[j]])==0) {
        type[j]=i
      }
    } 
  }
}

table(type)

#type
#1   2   3   4   5   7   8  11  14  17  28  31  32  33  39  45  59  67  69  96 134  
#17  56   4  40   4  21  28  16  17   1   7   4   1   1   7   9   5   4   2   2   4

pdf("proj_sim_250_8_topology.pdf",width = 10, height = 10)
par(mfrow=c(3,3))
plot(proj_trees[[2]],main="(56)",type = "cladogram",direction = "downwards", edge.width = 2)
plot(proj_trees[[4]],main="(40)",type = "cladogram",direction = "downwards", edge.width = 2)
plot(proj_trees[[8]],main="(28)",type = "cladogram",direction = "downwards", edge.width = 2)
plot(proj_trees[[7]],main="(21)",type = "cladogram",direction = "downwards", edge.width = 2)
plot(proj_trees[[1]],main="(17)",type = "cladogram",direction = "downwards", edge.width = 2)
plot(proj_trees[[14]],main="(17)",type = "cladogram",direction = "downwards", edge.width = 2)
plot(proj_trees[[11]],main="(16)",type = "cladogram",direction = "downwards", edge.width = 2)
plot(proj_trees[[45]],main="(9)",type = "cladogram",direction = "downwards", edge.width = 2)

plot(species_chrono,main="species",type = "cladogram",direction = "downwards", edge.width = 2)
dev.off()
#r^2########
library(lpSolveAPI)
library(parallel)
fermatweberdistance <- function(datamatrix) {
  n = dim(datamatrix)[1]
  m = dim(datamatrix)[2]
  lprec <- make.lp(0, n+m)
  objective = mat.or.vec(n+m,1)
  for (i in seq(n)) {
    objective[i] = 1
  }
  set.objfn(lprec, objective)
  for (i in seq(n)) {
    for (j in seq(m)) {
      for (k in seq(m)) {
        v = mat.or.vec(n+m,1)
        v[i] = 1
        v[n+k] = 1
        v[n+j] = -1
        add.constraint(lprec, v, ">=", datamatrix[i,k] - datamatrix[i,j])
      }
    }
  }
  solve(lprec)
  return((get.objective(lprec)))
}

r_proj_data<-matrix(unlist(label_output$`Projected points`),nrow=250,byrow = T)

sim_fermet<-fermatweberdistance(r_proj_data)
#130.2822

#sum of dist = 360.6831
r_sim_250_8<-label_output$`Sum of dist`/(label_output$`Sum of dist`+sim_fermet)
#0.7346407


#reload.data<-scan("rrrr.txt",what = "numeric",sep=" ")
#aaa<-matrix(as.numeric(reload.data),nrow=13,ncol=28,byrow=T)
#aaa.list<-lapply(seq_len(nrow(aaa)), function(i) aaa[i,])
#aaa.list.matrix<-lapply(aaa.list,make.matrix,
#                          n=8,tips=genes[[1]]$tip.label)
#aaa.trees<-lapply(aaa.list.matrix,upgma)
#class(aaa.trees)<-"multiPhylo"
#write.tree(aaa.trees,"extra_freq.newick")
#for(i in 1:8){
#  print(RF.dist(aaa.trees[[i]],species_chrono))
#}
